package org.jastadd.relast.compiler;

import org.jastadd.JastAddConfiguration;
import org.jastadd.option.ArgumentParser;
import org.jastadd.option.Option;

import java.util.ArrayList;

public abstract class AbstractCompiler {

  private final boolean jastAddCompliant;
  protected ArrayList<Option<?>> options;
  private String name;
  private ArgumentParser commandLine;

  private JastAddConfiguration configuration;

  public AbstractCompiler(String name, boolean jastaddCompliant) {
    this.name = name;
    this.jastAddCompliant = jastaddCompliant;
  }

  public JastAddConfiguration getConfiguration() throws CompilerException {
    if (configuration == null) {
      throw new CompilerException("Configuration only supported for JastAdd-compliant compilers!");
    }
    return configuration;
  }

  public int run(String[] args) throws CompilerException {

    options = new ArrayList<>();
    initOptions();
    if (jastAddCompliant) {
      configuration = new JastAddConfiguration(args, System.err, options);
    } else {
      commandLine = new ArgumentParser();
      commandLine.addOptions(options);
      commandLine.parseArgs(args, System.err);
    }
    return compile();
  }

  abstract int compile() throws CompilerException;

  protected void initOptions() {
    // there are no options by default
  }

  protected <OptionType extends Option<?>> OptionType addOption(OptionType option) {
    options.add(option);
    return option;
  }

  protected int error(String message) {
    System.err.println("Error: " + message);
    System.err.println();
    System.err.println("Usage: java -jar " + name + ".jar [--option1] [--option2=value] ...  <filename1> <filename2> ... ");
    System.err.println("Options:");
    commandLine.printHelp(System.err);
    return 1;
  }

  public String getName() {
    return name;
  }
}

